import { createAction, createReducer, on } from '@ngrx/store';
import { camelCaseToSentence, createTraitFactory } from 'ngrx-traits';
/**
 * Generates the typical ngrx code need to make a async action with
 * a request, success and failure actions, plus a status property to track its progress
 * and selectors to query the status.
 *
 * @param options - Config object for the trait factory
 * @param options.name - Name of the main request action, should be in camel case
 * @param options.actionProps - Optional param for the main request action, use the props()
 * function for its value, if not present action will have no params,
 * @param options.actionSuccessProps - Optional param for the request success action,
 * use the props() function for its value, if not present action success will have no params
 * @param options.actionFailProps - Optional param for the request fail action,
 * use the props() function for its value, if not present action fail will have no params
 * @returns the trait factory
 *
 * @example
 * // The following trait config
 * const traits = createEntityFeatureFactory(
 * {entityName: 'Todo'},
 * addAsyncActionTrait({
 *        name: 'createClient',
 *        actionProps: props<{ name: string }>(),
 *        actionSuccessProps: props<{ id: string }>(),
 *      }),
 * )({
 *      actionsGroupKey: 'Client',
 *      featureSelector: createFeatureSelector<AsyncActionState<'createClient'>>(
 *        'client',
 *      ),
 *    });
 * // will generate the actions and selectors
 * traits.actions.createClient({name:'Pedro'})
 * traits.actions.createClientSuccess({id:'123'})
 * traits.actions.createClientFail();
 * traits.selectors.isLoadingCreateClient
 * traits.selectors.isSuccessCreateClient
 * traits.selectors.isFailCreateClient
 */
export function addAsyncActionTrait({ name, actionProps, actionSuccessProps, actionFailProps, }) {
    const nameAsSentence = camelCaseToSentence(name);
    let internalActions;
    return createTraitFactory({
        key: name + '-call',
        config: {
            name,
            actionProps,
            actionSuccessProps,
            actionFailProps,
        },
        actions: ({ actionsGroupKey, }) => {
            internalActions = {
                request: (actionProps
                    ? createAction(`${actionsGroupKey} ${nameAsSentence}`, actionProps)
                    : createAction(`${actionsGroupKey} ${nameAsSentence}`)),
                requestSuccess: (actionSuccessProps
                    ? createAction(`${actionsGroupKey} ${nameAsSentence} Success`, actionSuccessProps)
                    : createAction(`${actionsGroupKey} ${nameAsSentence} Success`)),
                requestFail: (actionFailProps
                    ? createAction(`${actionsGroupKey} ${nameAsSentence} Failure`, actionFailProps)
                    : createAction(`${actionsGroupKey} ${nameAsSentence} Failure`)),
            };
            if (name) {
                return {
                    [`${name}`]: internalActions.request,
                    [`${name}Success`]: internalActions.requestSuccess,
                    [`${name}Fail`]: internalActions.requestFail,
                };
            }
            return internalActions;
        },
        selectors: () => {
            function isLoadingEntity(state) {
                return state[`${name}Status`] === 'loading';
            }
            function isSuccessEntity(state) {
                return state[`${name}Status`] === 'success';
            }
            function isFailEntity(state) {
                return state[`${name}Status`] === 'fail';
            }
            const capitalizedName = name.charAt(0).toUpperCase() + name.slice(1);
            return {
                [`isLoading${capitalizedName}`]: isLoadingEntity,
                [`isSuccess${capitalizedName}`]: isSuccessEntity,
                [`isFail${capitalizedName}`]: isFailEntity,
            };
        },
        initialState: ({ previousInitialState }) => previousInitialState,
        reducer: ({ initialState }) => {
            return createReducer(initialState, on(internalActions.request, (state) => (Object.assign(Object.assign({}, state), { [`${name}Status`]: 'loading' }))), on(internalActions.requestFail, (state) => (Object.assign(Object.assign({}, state), { [`${name}Status`]: 'fail' }))), on(internalActions.requestSuccess, (state) => (Object.assign(Object.assign({}, state), { [`${name}Status`]: 'success' }))));
        },
    });
}
//# sourceMappingURL=async-action.trait.js.map