import { entitiesPaginationTraitKey, } from './entities-pagination.model';
import { createPaginationTraitSelectors } from './entities-pagination.trait.selectors';
import { createPaginationTraitActions } from './entities-pagination.trait.actions';
import { loadEntitiesTraitKey, } from '../load-entities/load-entities.model';
import { createPaginationInitialState, createPaginationTraitReducer, } from './entities-pagination.trait.reducer';
import { createPaginationTraitEffects } from './entities-pagination.trait.effects';
import { createTraitFactory } from 'ngrx-traits';
import { createPaginationTraitMutators } from './entities-pagination.trait.mutators';
/**
 * Generates ngrx code to paginate an list of entities, this has 3 cache `cacheType`
 * - 'full': The full result is cache in memory and split in pages to render, useful
 * for small result but not so small that requires been render in pages
 * - 'partial': Backend returns partial results because is are to big, this has a cache a few pages forward
 * to avoid calling the backend on each page, the cache is clean when a new loadEntities is required
 * - 'grow': Similar to partial that the backend returns partial result, but in this case the cache grows,
 * after each loadEntities the cache is appended to the previous cache, this mode is ideal for infinite scrolls,
 * where you will only call loadNextPage.
 * To make the pagination experience smoother the loadEntities to get new pages is done when the current page is change to
 * the last cached page.
 * @param config
 * @param config.cacheType - Default to 'full', change the cache mode
 * @param config.pageSize - Default to 20, number of entities on  each page
 * @param config.currentPage - Default to 0, starting page
 * @param config.pagesToCache - Default to 3, used in partial and grow cache mode, is the number of
 * extra pages kept in cache to avoid calling the backend on each page
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,SingleSelectionState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addEntitiesPaginationTrait<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.loadTodosPage({index})
 * traits.actions.loadTodosPageSuccess()
 * traits.actions.loadTodosPageFail()
 * traits.actions.loadTodosNextPage()
 * traits.actions.loadTodosPreviousPage()
 * traits.actions.loadTodosFirstPage()
 * traits.actions.loadTodosLastPage()
 * traits.actions.clearTodosPagesCache()
 *
 * // page param is optional and defaults to current page
 * traits.selectors.isTodosPageInCache({page})
 * traits.selectors.selectPageTodosList({page})
 * traits.selectors.isLoadingTodosPage({page})
 * // prefer isLoadingTodosPage over isLoadingTodos which will return true even
 * // if the page loading is not the current one
 * traits.selectors.selectTodosPage({page})
 * traits.selectors.selectTodosPagedRequest()// use in effects to get paging parameter
 * traits.selectors.selectTodosPageInfo()
 */
export function addEntitiesPaginationTrait({ cacheType = 'full', pageSize = 20, currentPage = 0, pagesToCache = 3, } = {}) {
    return createTraitFactory({
        key: entitiesPaginationTraitKey,
        depends: [loadEntitiesTraitKey],
        config: {
            cacheType,
            pageSize,
            currentPage,
            pagesToCache,
        },
        actions: ({ actionsGroupKey, entitiesName }) => createPaginationTraitActions(actionsGroupKey, entitiesName),
        selectors: ({ previousSelectors, allConfigs, }) => createPaginationTraitSelectors(previousSelectors, allConfigs),
        mutators: ({ allSelectors, allConfigs }) => createPaginationTraitMutators(allSelectors, allConfigs),
        initialState: ({ previousInitialState, allConfigs, }) => createPaginationInitialState(previousInitialState, allConfigs),
        reducer: ({ initialState, allActions, allSelectors, allMutators, allConfigs, }) => createPaginationTraitReducer(initialState, allActions, allSelectors, allMutators, allConfigs),
        effects: ({ allActions, allSelectors }) => createPaginationTraitEffects(allActions, allSelectors),
    });
}
//# sourceMappingURL=entities-pagination.trait.js.map