import { createLoadEntitiesInitialState, createLoadEntitiesTraitReducer, } from './load-entities.trait.reducer';
import { loadEntitiesTraitKey, } from './load-entities.model';
import { createLoadEntitiesTraitMutators } from './load-entities.mutators';
import { createTraitFactory } from 'ngrx-traits';
import { createEntityAdapter } from '@ngrx/entity';
import { createLoadEntitiesTraitActions } from './load-entities.trait.actions';
import { createLoadEntitiesTraitSelectors } from './load-entities.trait.selectors';
/**
 * Generates the ngrx code needed to load a list of entities from the backend
 * with a load[EntitiesName], load[EntitiesName]Success, load[EntitiesName]Fail actions, and selectors to query the
 * entities like selectEntity[EntitiesName]List, selectEntity[EntitiesName]Ids, selectEntity[EntitiesName]Map, and its progress loading
 * with isLoading[EntitiesName],isLoading[EntitiesName]Success, isLoading[EntitiesName]Fail. This trait is the base for all other traits related
 * to a list of entities, the other will call loadEntities when needing data.
 * @param traitConfig - Config object fot the trait factory
 * @param traitConfig.selectId - Function that returns the id of an entity
 * @param traitConfig.sortComparer - Default sort function for to @ngrx/entity EntityAdapter
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends LoadEntitiesState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      {entityName: 'Todo'},
 *      addLoadEntitiesTrait<Todo>(),
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate the actions and selectors
 * traits.actions.loadTodos()
 * traits.actions.loadTodosSuccess({entities: todos})
 * traits.actions.loadTodosFail();
 * traits.selectors.selectTodosList
 * traits.selectors.selectTodosMap
 * traits.selectors.selectTodosIds
 * traits.selectors.selectTodosTotal
 * traits.selectors.isLoadingTodos
 * traits.selectors.isSuccessTodosSuccess
 * traits.selectors.isFailTodosFail
 */
export function addLoadEntitiesTrait(traitConfig) {
    const adapter = createEntityAdapter(traitConfig);
    return createTraitFactory({
        key: loadEntitiesTraitKey,
        config: Object.assign(Object.assign({}, traitConfig), { adapter }),
        actions: ({ actionsGroupKey, entitiesName }) => createLoadEntitiesTraitActions(actionsGroupKey, entitiesName),
        selectors: ({ allConfigs }) => createLoadEntitiesTraitSelectors(allConfigs),
        mutators: ({ allConfigs }) => createLoadEntitiesTraitMutators(allConfigs),
        initialState: ({ previousInitialState, allConfigs, }) => createLoadEntitiesInitialState(previousInitialState, allConfigs),
        reducer: ({ initialState, allActions, allMutators, allConfigs }) => createLoadEntitiesTraitReducer(initialState, allActions, allMutators, allConfigs),
    });
}
//# sourceMappingURL=load-entities.trait.js.map