import { capitalize, createTraitFactory } from 'ngrx-traits';
import { createReducer, on } from '@ngrx/store';
import { addAsyncActionTrait } from '../async-action/async-action.trait';
/**
 * Generates ngrx code needed to load and entity and store it in a state
 * @param entityName - Entity name, should be in camel case
 * @param options.actionProps - Optional param for the main request action,
 * use the props() function for its value, if not present action will have no params,
 * @param options.actionSuccessProps - Optional param for the request success
 * action, use the props() function for its value, if not present action success will have no params
 * @param options.actionFailProps - Optional param for the request fail action,
 * use the props() function for its value, if not present action fail will have no params
 * @returns the trait factory
 *
 * @example
 * const traits = createEntityFeatureFactory(
 * ...addLoadEntityTraits({
 *        entityName: 'client',
 *        requestProps: props<{ id: string }>(),
 *        responseProps: props<{ client: Client }>(),
 *      }),
 * )({
 *      actionsGroupKey: 'Client',
 *      featureSelector: createFeatureSelector<
 *        LoadEntityState<Client, 'client'>
 *        >('client'),
 *    });
 *
 * // will generate
 * traits.actions.loadClient({id:123});
 * traits.actions.loadClientSuccess({client: {id: '123', name: 'gabs'}});
 * traits.actions.loadClientFail();
 * traits.selectors.selectClient
 * traits.selectors.isLoadingLoadClient
 * traits.selectors.isSuccessLoadClient
 * traits.selectors.isFailLoadClient
 */
export function addLoadEntityTraits({ entityName, actionProps, actionSuccessProps, actionFailProps, }) {
    const capitalizedName = capitalize(entityName);
    return [
        addAsyncActionTrait({
            name: ('load' + capitalizedName),
            actionProps,
            actionSuccessProps,
            actionFailProps,
        }),
        createTraitFactory({
            key: `load${capitalizedName}`,
            config: { entityName, actionProps, actionSuccessProps, actionFailProps },
            selectors: () => {
                function selectEntity(state) {
                    return state[`${entityName}`];
                }
                return {
                    [`select${capitalizedName}`]: selectEntity,
                };
            },
            initialState: ({ previousInitialState, }) => previousInitialState,
            reducer: ({ initialState, allActions }) => {
                return createReducer(initialState, on(allActions[`load${capitalizedName}Success`], (state, action) => (Object.assign(Object.assign({}, state), { [entityName]: action[entityName] }))));
            },
        }),
    ];
}
//# sourceMappingURL=load-entity.traits.js.map