import { createSortTraitMutators } from './sort-entities.trait.mutators';
import { createSortInitialState, createSortTraitReducer, } from './sort-entities.trait.reducer';
import { createSortTraitSelectors } from './sort-entities.trait.selectors';
import { createSortTraitEffect } from './sort-entities.trait.effect';
import { sortTraitKey, } from './sort-entities.model';
import { loadEntitiesTraitKey, } from '../load-entities/load-entities.model';
import { createTraitFactory } from 'ngrx-traits';
import { createSortTraitActions } from './sort-entities.trait.actions';
/**
 * Generates ngrx code to sort locally or remotely a list of entities
 * @param config
 * @param config.defaultSort - Required, Default entity prop for the sort
 * @param config.remote - Optional field, default to false, when true disables local
 * sorting and every sort action call will now trigger a loadEntities action and the backend
 * should sort the data, use selectSort in the effect that call backend to get the requested sort,
 * when false all sorting is done in memory when the sort action is fired
 *
 * @example
 * // The following trait config
 * export interface TestState
 * extends EntityAndStatusState<Todo>, SortState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addSortEntitiesTrait<Todo>({
 *        defaultSort: {active:'id', direction:'asc'}
 *      })
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.sortTodos({active:'id', direction:'desc'})
 * traits.actions.resetTodosSort()
 *
 * traits.selectors.selectTodosSort()
 */
export function addSortEntitiesTrait({ remote = false, defaultSort, }) {
    return createTraitFactory({
        key: sortTraitKey,
        depends: [loadEntitiesTraitKey],
        config: { remote, defaultSort },
        actions: ({ actionsGroupKey, entitiesName }) => createSortTraitActions(actionsGroupKey, entitiesName),
        selectors: () => createSortTraitSelectors(),
        mutators: ({ allSelectors, allConfigs }) => createSortTraitMutators(allSelectors, allConfigs),
        initialState: ({ previousInitialState, allConfigs, }) => createSortInitialState(previousInitialState, allConfigs),
        reducer: ({ initialState, allActions, allMutators, allConfigs }) => createSortTraitReducer(initialState, allActions, allMutators, allConfigs),
        effects: ({ allActions, allConfigs }) => createSortTraitEffect(allActions, allConfigs),
    });
}
//# sourceMappingURL=sort-entities.trait.js.map