import { EntitiesPaginationConfig, EntitiesPaginationSelectors } from './entities-pagination.model';
/**
 * Generates ngrx code to paginate an list of entities, this has 3 cache `cacheType`
 * - 'full': The full result is cache in memory and split in pages to render, useful
 * for small result but not so small that requires been render in pages
 * - 'partial': Backend returns partial results because is are to big, this has a cache a few pages forward
 * to avoid calling the backend on each page, the cache is clean when a new loadEntities is required
 * - 'grow': Similar to partial that the backend returns partial result, but in this case the cache grows,
 * after each loadEntities the cache is appended to the previous cache, this mode is ideal for infinite scrolls,
 * where you will only call loadNextPage.
 * To make the pagination experience smoother the loadEntities to get new pages is done when the current page is change to
 * the last cached page.
 * @param config
 * @param config.cacheType - Default to 'full', change the cache mode
 * @param config.pageSize - Default to 20, number of entities on  each page
 * @param config.currentPage - Default to 0, starting page
 * @param config.pagesToCache - Default to 3, used in partial and grow cache mode, is the number of
 * extra pages kept in cache to avoid calling the backend on each page
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,SingleSelectionState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addEntitiesPaginationTrait<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.loadTodosPage({index})
 * traits.actions.loadTodosPageSuccess()
 * traits.actions.loadTodosPageFail()
 * traits.actions.loadTodosNextPage()
 * traits.actions.loadTodosPreviousPage()
 * traits.actions.loadTodosFirstPage()
 * traits.actions.loadTodosLastPage()
 * traits.actions.clearTodosPagesCache()
 *
 * // page param is optional and defaults to current page
 * traits.selectors.isTodosPageInCache({page})
 * traits.selectors.selectPageTodosList({page})
 * traits.selectors.isLoadingTodosPage({page})
 * // prefer isLoadingTodosPage over isLoadingTodos which will return true even
 * // if the page loading is not the current one
 * traits.selectors.selectTodosPage({page})
 * traits.selectors.selectTodosPagedRequest()// use in effects to get paging parameter
 * traits.selectors.selectTodosPageInfo()
 */
export declare function addEntitiesPaginationTrait<Entity>({ cacheType, pageSize, currentPage, pagesToCache, }?: EntitiesPaginationConfig): import("ngrx-traits").TraitFactory<import("../load-entities/load-entities.model").LoadEntitiesState<Entity> & import("./entities-pagination.model").EntitiesPaginationState, import("./entities-pagination.model").EntitiesPaginationActions, EntitiesPaginationSelectors<Entity>, {}, "pagination", EntitiesPaginationConfig, import("ngrx-traits").AllTraitConfigs>;
