import { FilterEntitiesConfig, FilterEntitiesSelectors } from './filter-entities.model';
/**
 * Generates the ngrx code needed to filter a list of entities locally or remotely, adds a filter
 * action and a selectFilter selector, the filter action is debounced and the filter will only
 * call the loadEntities action if the params have changed, so there is no need to implement that in
 * the components. The filter action has a `forceLoad` param which can
 * be use to skip that restriction for one call or setting the `defaultDebounceTime` to 0 for all calls.
 * Calling the filter action will also replace the `filters` param in the store, if the `patch` param is set
 * the filters are merged with the previous value in the store.
 * @param traitConfig - Config object fot the trait factory
 * @param traitConfig.defaultFilter - Initial value for the filter
 * @param traitConfig.filterFn - Function to filter entities in memory, if not present then its expected
 * is filtered by the backend
 * @param traitConfig.defaultDebounceTime - Value in milliseconds. Default to 400ms
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,FilterState<TodoFilter>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      //addFilterEntitiesTrait<Todo,TodoFilter>() //  remote filtering
 *      addFilterEntitiesTrait<Todo,TodoFilter>({filterFn: (filter, entity) => // local filtering
 *          filter?.content && entity.content?.includes(filter?.content) || false})// remote
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.filterTodos()
 * traits.selectors.selectTodosFilter()
 */
export declare function addFilterEntitiesTrait<Entity, F>({ defaultDebounceTime, defaultFilter, filterFn, }?: FilterEntitiesConfig<Entity, F>): import("ngrx-traits").TraitFactory<import("../load-entities/load-entities.model").LoadEntitiesState<Entity> & import("./filter-entities.model").FilterEntitiesState<F>, import("./filter-entities.model").FilterEntitiesActions<F>, FilterEntitiesSelectors<Entity, F>, import("./filter-entities.model").FilterEntitiesMutators<Entity, F>, "filter", FilterEntitiesConfig<Entity, F>, import("ngrx-traits").AllTraitConfigs>;
